# Generated by Django 2.2 on 2019-04-30 21:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0005_issue_dek'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='issue',
            name='order',
        ),
        migrations.AddField(
            model_name='candidate',
            name='active',
            field=models.BooleanField(null=True),
        ),
        migrations.AddField(
            model_name='candidate',
            name='date_exited',
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name='category',
            name='status',
            field=models.CharField(choices=[('draft', 'Draft'), ('live', 'Live')], default='draft', max_length=5),
        ),
        migrations.AddField(
            model_name='issue',
            name='status',
            field=models.CharField(choices=[('draft', 'Draft'), ('live', 'Live')], default='draft', max_length=5),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-07 16:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0010_story_date_added'),
    ]

    operations = [
        migrations.AlterField(
            model_name='story',
            name='date_added',
            field=models.DateField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2 on 2019-04-30 15:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0004_auto_20190429_1423'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='dek',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.utils import timezone
from rest_framework.decorators import action
from .base import TokenAuthedViewSet
from issuestracker.models import Category
from issuestracker.serializers import (
    CategorySerializer,
    CategoryListSerializer,
)


class CategoryViewset(TokenAuthedViewSet):
    queryset = Category.objects.all()
    lookup_field = "slug"
    pagination_class = None
    throttle_classes = []

    def get_serializer_class(self):
        if hasattr(self, "action") and self.action == "list":
            return CategoryListSerializer
        return CategorySerializer

    @action(detail=True, methods=["post"])
    def publish(self, request, *args, **kwargs):
        slug = request.data.get("slug")

        update = self.update(request, *args, **kwargs)

        print("Publish too!")

        c = Category.objects.get(slug=slug)
        c.last_published = timezone.now()
        c.save()

        return update



from rest_framework import serializers
from issuestracker.models import Category, Issue


class CandidateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = ("slug",)


class IssueSerializer(serializers.ModelSerializer):
    class Meta:
        model = Issue
        fields = ("slug",)


class CategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = ("slug",)



from .base import BakeryBase
from rest_framework.response import Response
from issuestracker.models import Category
from issuestracker.serializers.bakery.CategoryPage import CategorySerializer


class BakeryCategory(BakeryBase):
    def get(self, request, category=None):
        return Response(
            CategorySerializer(
                Category.live.get(slug=self.kwargs["category"])
            ).data
        )



from .base import BakeryBase
from rest_framework.response import Response
from issuestracker.models import Category
from issuestracker.serializers.bakery.HomePage import CategorySerializer


class BakeryHome(BakeryBase):
    def get(self, request, format=None):
        return Response(
            CategorySerializer(Category.live.all(), many=True).data
        )



from issuestracker.models import Issue
from rest_framework import serializers
from .Position import PositionSerializer
from .Faq import FaqSerializer
from .Story import StorySerializer


class IssueSerializer(serializers.ModelSerializer):
    position_set = PositionSerializer(many=True, read_only=True)
    faq_set = FaqSerializer(many=True, read_only=True)
    story_set = StorySerializer(many=True, read_only=True)

    class Meta:
        model = Issue
        fields = (
            "id",
            "name",
            "slug",
            "dek",
            "summary",
            "category",
            "position_set",
            "faq_set",
            "story_set",
            "status",
            "last_updated",
            "last_published",
        )


class IssueListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Issue
        fields = ("name", "slug", "dek", "category")



from rest_framework import serializers
from issuestracker.models import Faq, Story, Position, Issue, Candidate


class PositionSerializer(serializers.ModelSerializer):
    candidates = serializers.SerializerMethodField()

    def get_candidates(self, obj):
        return [candidate.slug for candidate in obj.candidates.all()]

    class Meta:
        model = Position
        fields = ("name", "explanation", "candidates")


class FaqSerializer(serializers.ModelSerializer):
    class Meta:
        model = Faq
        fields = ("question", "answer")


class StorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Story
        fields = ("headline", "link")


class IssueSerializer(serializers.ModelSerializer):
    category = serializers.SerializerMethodField()
    category_slug = serializers.SerializerMethodField()
    category_icon = serializers.SerializerMethodField()
    faq_set = FaqSerializer(many=True, read_only=True)
    story_set = StorySerializer(many=True, read_only=True)
    position_set = PositionSerializer(many=True, read_only=True)
    candidates_with_position_count = serializers.SerializerMethodField()
    candidates_count = serializers.SerializerMethodField()

    def get_category(self, obj):
        return str(obj.category)

    def get_category_slug(self, obj):
        return obj.category.slug

    def get_category_icon(self, obj):
        return obj.category.icon

    def get_candidates_with_position_count(self, obj):
        count = 0
        for position in obj.position_set.all():
            count += position.candidates.count()

        return count

    def get_candidates_count(self, obj):
        return Candidate.objects.all().count()

    class Meta:
        model = Issue
        fields = (
            "name",
            "slug",
            "category",
            "category_slug",
            "category_icon",
            "summary",
            "faq_set",
            "story_set",
            "position_set",
            "candidates_with_position_count",
            "candidates_count",
        )



from .base import ReadOnlyTokenAuthedViewSet
from rest_framework.decorators import action
from rest_framework.response import Response
from issuestracker.models import Candidate
from issuestracker.serializers import CandidateSerializer


class CandidateViewset(ReadOnlyTokenAuthedViewSet):
    queryset = Candidate.objects.all()
    serializer_class = CandidateSerializer
    lookup_field = "slug"
    pagination_class = None
    throttle_classes = []

    @action(detail=False, methods=["get"])
    def by_slug(self, request, *args, **kwargs):
        output = {}
        for candidate in self.queryset:
            output[candidate.slug] = CandidateSerializer(candidate).data
        return Response(output)



from issuestracker.models import Update
from rest_framework import serializers


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Update
        fields = ("text", "issue", "created_on")



from django.db import models


class Story(models.Model):
    """
    Related coverage about a given issue.
    """

    headline = models.CharField(max_length=200)
    link = models.URLField(max_length=200)
    issue = models.ForeignKey("Issue", on_delete=models.PROTECT)
    date_added = models.DateField(null=True, blank=True)
    order = models.IntegerField()

    def __str__(self):
        return self.headline

    class Meta:
        ordering = ["order"]



from django.db import models


class Position(models.Model):
    """
    A specific position held by candidates on an issue (e.g. "For A 10%
    Carbob Tax", "Against Medicare For All", etc.)
    """

    name = models.CharField(max_length=200)
    explanation = models.TextField(null=True, blank=True)
    issue = models.ForeignKey("Issue", on_delete=models.PROTECT)
    candidates = models.ManyToManyField("Candidate")
    order = models.IntegerField()

    def __str__(self):
        return self.name

    class Meta:
        unique_together = ("name", "issue")
        ordering = ["order"]



# Generated by Django 2.2 on 2019-04-30 22:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0006_auto_20190430_2144'),
    ]

    operations = [
        migrations.AlterField(
            model_name='candidate',
            name='date_exited',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from rest_framework import serializers
from issuestracker.models import Category, Issue


class IssueSerializer(serializers.ModelSerializer):
    class Meta:
        model = Issue
        fields = ("name", "slug")


class CategorySerializer(serializers.ModelSerializer):
    issue_set = IssueSerializer(many=True, read_only=True)

    class Meta:
        model = Category
        fields = ("name", "slug", "issue_set")



# Generated by Django 2.2 on 2019-04-30 22:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('issuestracker', '0007_auto_20190430_2202'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='candidate',
            name='headshot',
        ),
    ]



from .base import BakeryBase
from rest_framework.response import Response
from issuestracker.models import Category
from issuestracker.serializers.bakery.CandidatePage import CategorySerializer


class BakeryCandidate(BakeryBase):
    def get(self, request, candidate=None):
        issues = CategorySerializer(
            Category.live.all(), context={"slug": candidate}, many=True
        ).data

        for category in issues:
            category["issue_set"] = [
                issue
                for issue in category["issue_set"]
                if issue["position"] is not None
            ]

        return Response(
            {
                "slug": candidate,
                "categories": [
                    category
                    for category in issues
                    if len(category["issue_set"]) > 0
                ],
            }
        )



from django.views.generic import TemplateView
from django.urls import reverse_lazy
from issuestracker.conf import settings as app_settings
from issuestracker.utils.auth import secure


@secure
class Base(TemplateView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["user"] = self.request.user
        context["API_TOKEN"] = app_settings.SECRET_KEY

        context["API_ROOT"] = reverse_lazy("issuestracker:api-root")

        return context



from .base import BakeryBase
from rest_framework.response import Response
from issuestracker.models import Issue
from issuestracker.serializers.bakery.IssuePage import IssueSerializer


class BakeryIssue(BakeryBase):
    def get(self, request, issue=None):
        return Response(
            IssueSerializer(Issue.live.get(slug=self.kwargs["issue"])).data
        )



from django.db import models


class Candidate(models.Model):
    """
    Candidates with positions on issues.
    """

    uid = models.CharField(max_length=250, unique=True)
    slug = models.SlugField(max_length=50, unique=True)
    fec_uid = models.CharField(
        max_length=50, null=True, blank=True, unique=True
    )
    name = models.CharField(max_length=100)
    active = models.BooleanField(null=True)
    date_exited = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return self.name



from django.contrib import admin
from .models import Candidate, Category, Faq, Issue, Position, Story, Update

admin.site.register(Candidate)
admin.site.register(Category)
admin.site.register(Faq)
admin.site.register(Issue)
admin.site.register(Position)
admin.site.register(Story)
admin.site.register(Update)



from rest_framework.views import APIView
from issuestracker.utils.api_auth import TokenAPIAuthentication
from issuestracker.models import Candidate
from issuestracker.serializers.Candidate import CandidateSerializer


class BakeryBase(APIView):
    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = []

    def get_all_candidates(self):
        output = {}
        for candidate in Candidate.objects.all():
            output[candidate.slug] = CandidateSerializer(candidate).data
        return output
